package com.collabnet.ctf.ejb;

import java.util.Date;

import javax.inject.Inject;

import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.transaction.api.annotation.Transactional;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.collabnet.ctf.common.server.user.domain.User;
import com.collabnet.ctf.common.server.user.domain.UserLogin;
import com.collabnet.ctf.ejb.user.domain.UserLoginRepository;
import com.collabnet.ctf.ejb.user.domain.UserRepository;

/**
 * CollabNet TeamForge(r) Copyright 2007-2013 CollabNet, Inc. All rights
 * reserved. http://www.collab.net
 */
@RunWith(Arquillian.class)
public class JPAUserLoginRepositoryTest extends JPATest {

	@Inject
	private UserLoginRepository userLoginRepository;

	@Inject
	private UserRepository userRepository;

	private UserLogin myUserLogin;

	@Override
	public void setUp() throws Exception {
		super.setUp();

	}

	@Override
	public void tearDown() throws Exception {
		super.tearDown();

	}

	/**
	 * Method to test persistence of creation of the user login entity
	 * 
	 * @throws Throwable
	 *             if anything goes wrong
	 */
	@Test
	@Transactional
	public void testCreate() throws Throwable {

		Date now = new Date();
		int id = getRandomId();
		createUserLogin(userLoginRepository, id, now, now, 10);

		validateUserLogin(id, now, now, 10, myUserLogin);

	}

	/**
	 * Method to test findByUserSurrogateId
	 * 
	 * @throws Throwable
	 *             if anything goes wrong
	 */
	@Test
	@Transactional
	public void testFindByUserSurrogateId() throws Throwable {

		Date now = new Date();
		int id = getRandomId();
		createUserLogin(userLoginRepository, id, now, now, 10);

		UserLogin userLogin = userLoginRepository.findByUserSurrogateId(id);
		assertNotNull(userLogin);

	}

	/**
	 * Method to test findByUserSurrogateId
	 * 
	 * @throws Throwable
	 *             if anything goes wrong
	 */
	@Test
	@Transactional
	public void testFindByUsername() throws Throwable {

		String userName = getRandomSfName("usr");
		User user = userRepository.create(userName, "john@cn.net",
				"John Smith", "CN", "US", null, false, false, "ACTIVE", "ALM",
				null, "user1001");

		Date now = new Date();

		createUserLogin(userLoginRepository, user.getSurrogateId(), now, now,
				10);

		UserLogin userLogin = userLoginRepository.findByUsername(user
				.getUsername());
		assertNotNull(userLogin);

	}

	/**
	 * Creates a user login entity
	 * 
	 * @param userLoginRepository
	 *            the user login repository object
	 * @param userSurrogateId
	 *            the user surrogate id
	 * @param lastLogin
	 *            the last login date
	 * @param previousLastLogin
	 *            the previous last login date
	 * @param count
	 *            the failed user login count
	 */
	private void createUserLogin(UserLoginRepository userLoginRepository,
			int userSurrogateId, Date lastLogin, Date previousLastLogin,
			int count) {
		myUserLogin = userLoginRepository.create(userSurrogateId, lastLogin,
				previousLastLogin, count);

	}

	/**
	 * Validates that the values set in the UserLogin bean match what is
	 * expected
	 * 
	 * @param userSurrogateId
	 *            the user surrogate id
	 * @param lastLogin
	 *            the last login date
	 * @param previousLastLogin
	 *            the previous last login date
	 * @param count
	 *            the failed user login count
	 * @param userLogin
	 *            the userLogin bean
	 */
	private void validateUserLogin(int userSurrogateId, Date lastLogin,
			Date previousLastLogin, int count, UserLogin userLogin) {
		assertEquals(userSurrogateId, userLogin.getUserSurrogateId());
		assertEquals(count, userLogin.getFailedLoginCount());
		assertEquals(lastLogin, userLogin.getLastLogin());
		assertEquals(previousLastLogin, userLogin.getPreviousLastLogin());

	}

}
